/**
  ******************************************************************************
  * @file           : main.c
  * @author         : sinomcu.com-AE
  * @brief          : Main program body
  *  
  *          This file provides example code for Power stop mode WFE and printf by uart:
  * Run uart DMA receive and send 5S,than enter stop LDO low power mode,USART receive wakeup core.
  * For details, see "readme.txt"
  *         
  ******************************************************************************
   * @attention 
  *                           DISCLAIMER
  * The present firmware which is for guidance only aims at providing customers
  * with coding information regarding their products in order for them to save
  * time. As a result, sinomcu.com shall not be held liable for any direct,indirect
  * or consequential damages with respect to any claims arising from the content 
  * of such firmware and/or the use made by customers of the coding information 
  * contained herein in connection with their products.
  *
  * <h2><center>&copy; Copyright (c) 2021 sinomcu.com All rights reserved.</center></h2>
  *
  ******************************************************************************
  */  
  
#include "system_define.h"

#define COUNT_PERIOD              500       // count period in ms
#define ENTER_STOP_DELAY_COUNT    10        // ENTER_STOP_DELAY_COUNT * COUNT_PERIOD

extern FlagStatus UART_IDIE_Flag,DMA_RecvComplete_Flag,DMA_SendComplete_Flag,DMA_TransferError_Flag;
uint8_t uart_recv_num=0;

int main(void) {
  __IO uint32_t run_count = 0;
  uint8_t stop_enter_count = 0;

  StopLDOLP_Wake_Set();

#if(defined(SYSCLK_USE_PLL_48M) && SYSCLK_USE_PLL_48M) 
  sysclk_init();                            // sysclk 48M,HSI 8M /2*12;if not call this function,default HSI 8M
#endif
  SystemCoreClockUpdate();                  // get System Core Clock
  
  sys_tick_init();
  gpio_init();
  dma_uart_init();
#if(defined(SYSCLK_USE_PLL_48M) && SYSCLK_USE_PLL_48M) 
  MS32_RCC_SetUSARTClockSource(MS32_RCC_USART1_CLKSOURCE_SYSCLK);    // uart clk sysclk ,TX DMA uart 
  usart1_uart_init(UART_BAUD_RATE_VALUE);  
#else
  MS32_RCC_SetUSARTClockSource(MS32_RCC_USART1_CLKSOURCE_HSI);       // uart clk HSI 8M   
  usart1_uart_init(UART_BAUD_RATE_VALUE); 
#endif
  LED1_ON(); 
  printf("\r\n\r\n*****Power STOP WFE Exit USART Example*****");
  printf("\r\n--Inf:System Core Clock %d Hz.",SystemCoreClock); 

  MS32_USART_SetWKUPType(USART1, MS32_USART_WAKEUP_ON_STARTBIT);
  MS32_USART_EnableInStopMode(USART1);                               // set UESM 

  Start_User_MS_SF_Timer(COUNT_PERIOD);
  while(1) {
    if(DMA_RecvComplete_Flag == SET || UART_IDIE_Flag == SET) {      // DMA Uart Recv Complete
      if(DMA_RecvComplete_Flag == SET ) {
        DMA_RecvComplete_Flag = RESET;
        MS32_DMA_Restart(DMA1, MS32_DMA_CHANNEL_2, UART_BUFF_LEN);   // start USART send 
        while(!(MS32_USART_IsActiveFlag_TC(USART1))){ ; }            // DMA_RecvComplete_Flag && UART_IDIE_Flag
      }
      if(UART_IDIE_Flag == SET) {
        UART_IDIE_Flag = RESET;
        uart_recv_num = UART_BUFF_LEN - MS32_DMA_GetDataLength(DMA1,MS32_DMA_CHANNEL_3);
        MS32_DMA_Restart(DMA1, MS32_DMA_CHANNEL_2, uart_recv_num);   // start USART send 
        MS32_DMA_Restart(DMA1,MS32_DMA_CHANNEL_3, UART_BUFF_LEN);
      }
    }

    if( DMA_SendComplete_Flag == SET) {               // DMA Uart Send Complete
      DMA_SendComplete_Flag = RESET;
    }
    if(DMA_TransferError_Flag == SET) {
      DMA_TransferError_Flag = RESET;                 // DMA transfer ERROR LED blink fast
      printf("\r\n##Err:DMA Transfer ERROR!");
    }

    if(Get_State_User_MS_SF_Timer() == SF_TIMER_OV) {
      Start_User_MS_SF_Timer(COUNT_PERIOD);                          // clear Software timer OV
      LED1_TOGGLE();
      run_count++;
      printf("\r\n--Inf: running count:%d ",run_count);

      stop_enter_count++;
      if(stop_enter_count == ENTER_STOP_DELAY_COUNT) {               // enter low power count 5S
        stop_enter_count = 0;
        printf("\r\n--Inf:Enter STOP WFE;Please Send Data Exit Low Power!");
        LED1_OFF(); 
#if(defined(SYSCLK_USE_PLL_48M) && SYSCLK_USE_PLL_48M) 
        MS32_USART_Disable(USART1);
        MS32_RCC_SetUSARTClockSource(MS32_RCC_USART1_CLKSOURCE_HSI);      // USART rx up core,need USART CLOCK use HSI  
        usart1_uart_init(UART_BAUD_RATE_VALUE);
#endif
        //while(MS32_USART_IsActiveFlag_BUSY(USART1)) { ; }            // make sure USART1 not busy
        MS32_USART_DisableDMAReq_RX(USART1);                         // for USART up core from STOP,disable DMA_RX

        LED2_ON();
        MS32_PWR_EnterSTOPMode(PWR_LOWPOWERREGULATOR_ON,PWR_STOPENTRY_WFE);    //enter STOP 
        LED2_OFF();                                                  // exit low power mode LED2 OFF

#if(defined(SYSCLK_USE_PLL_48M) && SYSCLK_USE_PLL_48M) 
        sysclk_init();                                               // stop mode need reinit clock,if sysclock not HSI
        MS32_USART_Disable(USART1);
        MS32_RCC_SetUSARTClockSource(MS32_RCC_USART1_CLKSOURCE_SYSCLK);        // uart clk sys 48M
        usart1_uart_init(UART_BAUD_RATE_VALUE);
#else
        MS32_USART_EnableDMAReq_RX(USART1); 
#endif
        printf("\r\n--Inf:Wake Up.");                                
     }
    }
  }
}

/**
  * @brief  SystemClock_Config HSI/2 to PLL*12 sysclk 48M
  * @param  None
  * @retval None
  */
void sysclk_init(void) {
  MS32_RCC_OscInitTypeDef osc_cfg;
  MS32_RCC_PllInitTypeDef pll_cfg;
  MS32_RCC_ClkInitTypeDef clk_cfg;

  // get and clear reset flag 
  MS32_RCC_GetResetFlag(ENABLE);
  
  // osc config 
  osc_cfg.HSIState = MS32_RCC_HSI_ON;
  osc_cfg.HSEState = MS32_RCC_HSE_OFF;
  osc_cfg.LSIState = MS32_RCC_LSI_OFF;
  osc_cfg.LSEState = MS32_RCC_LSE_OFF;
  MS32_RCC_OscConfig(&osc_cfg);
  
  // pll config 
  pll_cfg.PLLState  = MS32_RCC_PLL_ON;
  pll_cfg.PLLSource = MS32_RCC_PLL_SOURCE_HSI_DIV_2;       // HSI/2 4M
  pll_cfg.PLLMUL    = MS32_RCC_PLL_MUL12;
  pll_cfg.PREDIV    = MS32_RCC_PLL_PREDIV_DIV1;
  MS32_RCC_PllConfig(&pll_cfg);
  
  // sysclk config 
  clk_cfg.SYSCLKSource   = MS32_RCC_SYSCLK_SOURCE_PLL;
  clk_cfg.AHBCLKDivider  = MS32_RCC_SYSCLK_DIV1;
  clk_cfg.APB1CLKDivider = MS32_RCC_HCLK_DIV1;
  MS32_RCC_ClockConfig(&clk_cfg);
}

/******************************** END OF FILE *********************************/
